{% extends "base.html" %}
{% block content %}        
<section id="content">
<ol id="post-list">
{% for article in articles %}        
        <li><article class="hentry">    
                <header> <h2 class="entry-title"><a href="{{ article.url }}" rel="bookmark" title="Permalink to {{ article.title}}">{{ article.title }}</a></h2> </header>
                <footer class="post-info">
                    <abbr class="published" title="{{ article.date.isoformat() }}"> {{ article.date.strftime('%Y-%m-%d %H:%M') }} </abbr>
                    {% if article.author %}<address class="vcard author">By <a class="url fn" href="#">{{ article.author }}</a></address>{% endif %}
                </footer><!-- /.post-info -->
                <div class="entry-content"> {{ article.summary }} </div><!-- /.entry-content -->
        </article></li>
{% endfor %}
</ol><!-- /#posts-list -->
</section><!-- /#content -->
{% endblock content %}
